<?php

if (!isset($_SESSION["team"]))
	require_once "server_files/403.php";

/**
 * @var Team $team
 * @var Tournament $tournament
 */
$team = $_SESSION["team"];
$tournament = Tournament::fromId($team->getTournamentId());

if (isset($_POST["send_synthesis"])) {
	$save_synthesis = new SaveSynthesis();
	try {
		$save_synthesis->makeVerifications();
		$save_synthesis->saveSynthesis();
	} catch (AssertionError $e) {
		$has_error = true;
		$error_message = $e->getMessage();
	}
}

$syntheses = $tournament->getAllSyntheses($team->getId());
$syntheses_final = null;
if ($team->isSelectedForFinal())
	$syntheses_final = $FINAL->getAllSyntheses($team->getId());

class SaveSynthesis
{
	private $dest;
	private $round;
	private $file;

	public function __construct()
	{
		$this->file = $_FILES["synthese"];
		$this->round = htmlspecialchars($_POST["round"]);
		$this->dest = DestType::fromName(strtoupper(htmlspecialchars($_POST["dest"])));
	}

	public function makeVerifications()
	{
		global $LOCAL_PATH, $tournament;

        ensure($this->dest != DestType::DEFENSEUR, "La source est invalide.");
        ensure($this->round == 1 || $this->round == 2, "Le tour est invalide.");
        $now = date("Y-m-d H:i");
        ensure($this->round == 1 && $now < $tournament->getSynthesesDate() || $this->round == 2 && $now < $tournament->getSynthesesDate2(), "Vous ne pouvez plus rendre de note de synthèse pour le tour $this->round.");
		ensure($this->file["size"] <= 2e6, "Le fichier doit peser moins que 2 Mo.");
		ensure(!$this->file["error"], "Une erreur est survenue.");
		ensure(finfo_file(finfo_open(FILEINFO_MIME_TYPE), $this->file["tmp_name"]) == "application/pdf", "Le fichier doit être au format PDF.");
		ensure(is_dir("$LOCAL_PATH/files") || mkdir("$LOCAL_PATH/files"), "Un problème est survenue dans l'envoi du fichier. Veuillez contacter l'administrateur du serveur.");
	}

	public function saveSynthesis()
	{
		global $LOCAL_PATH, $DB, $team, $tournament, $FINAL;
		do
			$id = genRandomPhrase(64);
		while (file_exists("$LOCAL_PATH/files/$id"));

		if (!rename($this->file["tmp_name"], "$LOCAL_PATH/files/$id"))
			throw new AssertionError("Une erreur est survenue lors de l'envoi du fichier.");

		$req = $DB->prepare("INSERT INTO `syntheses`(`file_id`, `team`, `tournament`, `round`, `dest`) VALUES (?, ?, ?, ?, ?);");
		$req->execute([$id, $team->getId(), $team->isSelectedForFinal() ? $FINAL->getId() : $tournament->getId(), $this->round, $this->dest]);

		return false;
	}
}

require_once "server_files/views/syntheses.php";
